/***************************************************************************
                          enemigos.cpp  -  description
                             -------------------
    begin                : Wed Mar 7 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "enemigos.h"
#include "niveles.h"
#include "controlador.h"

Enemigos::Enemigos(Punto2D _pos, Niveles *niv, Controlador *c, int _estado, int _pattern):Objetos(niv, c, -1)
{
	explode = true;
	radio_choque = 24;
	life = ENEMY_LIFE;
	damage = 15;
	pos = _pos;
	vel = Punto2D(0, -1);
	img_cod = Gra_LoadImage("Enemigos/base.png", true);

	estado = _estado;
	calentamiento = ENEMY_SHOT;
	def_pattern = _pattern;

	vueltas = 4;
	angulo = 0;
	dir = (pos.X()<(Gra_ScreenWidth()/2))?(1):(-1);
	puntos = 33;

	control->AddObjetoMalo(this);
}

Enemigos::~Enemigos()
{
}

void Enemigos::Mover()
{
	Punto2D tmp = current->ScreenPos(pos);
	
	/* Si todavia no entro en pantalla no hago nada */
	if (tmp.Y() < 0) return;

	if (estado == STATE_EXIT)  /* Ya me estoy llendo !! ... solo avanzo !!! */
	{
		vel = Punto2D(0, -1);
		pos += vel*Gra_SpeedFactor();
		return;
	}

	Punto2D h_pos = control->getHumanoCerca(pos);

	/* Si no estoy haciendo nada ... */
	if (estado == STATE_NORMAL) {
		if (h_pos.dist(pos) < 300) estado = STATE_ATTACK; /* jeje, humano cerca :-), you will die!! */
		if (h_pos == Punto2D(0,0)) estado = STATE_EXIT; /* ABURRIDO!!, no hay nadie quien matar!! */
	}

	/* Veo que tengo que hacer */
	switch (estado)
	{
		case (STATE_NORMAL): /* Avanzar */
			pos += vel*Gra_SpeedFactor();
			break;
		case (STATE_RANDOM): /* Locura, hace lo que quiere */
			Random();
			break;
		case (STATE_ATTACK): /* Araco */
			Attack();
			break;
		case (STATE_PATTERN): /* Sigo un patron de movimiento */
			Pattern();
			break;
	}

	/* Verifico si sali de pantalla => "me muero" */
	tmp = current->ScreenPos(pos);
	if (tmp.Y() > Gra_ScreenHeight())
		life = -1;
}
	
void Enemigos::Dibujar()
{
	Punto2D pos_tmp = current->ScreenPos(pos);
	Gra_BlitImagen(img_cod, NULL, pos_tmp);
}

void Enemigos::Fire(Punto2D h_pos)
{
	/* Verifico si tengo a tiro a un humano */
	if (h_pos == Punto2D(0,0)) return;
	if (!calentamiento)
	{
		if (h_pos.Y() > pos.Y())
			return;
		Punto2D tmp = pos.getDir( h_pos );
		tmp *= (radio_choque+5);
		tmp += pos;
		new Faser(tmp, h_pos, current, control, true);
		calentamiento = ENEMY_SHOT;
	}
	else
		calentamiento--;
}

void Enemigos::Pattern()
{
	switch (def_pattern)
	{
		case (MOVE_CIRCLE):
			Move_Circle();
			break;
		case (MOVE_DIAGONAL):
			Move_Diagonal();
			break;
	}
}

void Enemigos::Attack()
{
	Punto2D h_pos;
	h_pos = control->getHumanoCerca(pos);
	if (h_pos == Punto2D(0,0))
		return;

	/* Me muevo en direccion al humano */
	vel = pos.getDir( h_pos );
	
	/* si esta cerca disparo */
	if (pos.dist( h_pos ) < 200)
		Fire(h_pos);

	pos += vel*Gra_SpeedFactor();
}

void Enemigos::Random()
{
}

void Enemigos::Move_Circle()
{
	float ca, sa, x, y;
	Punto2D h_pos;

	h_pos = control->getHumanoCerca(pos);
	if (pos.dist( h_pos ) < 250)
			Fire(h_pos);

	ca = cos(dir*0.02*Gra_SpeedFactor());
	sa = sin(dir*0.02*Gra_SpeedFactor());

	x = vel.X();
	y = vel.Y();

	/* Tal vez un Punto2D->rotar seria util :-) */
	vel.setX( x * ca - y * sa );
	vel.setY( x * sa + y * ca );

	vel /= vel.Norma();
	vel *= 2;
	pos += vel*Gra_SpeedFactor();
	angulo += 0.02*Gra_SpeedFactor();
	if (angulo >= (2*PI))
	{
		angulo = 0;
		vueltas--;
		if (vueltas == 0)
			estado = STATE_EXIT;
	}
}

void Enemigos::Move_Diagonal()
{
	Punto2D h_pos;

	h_pos = control->getHumanoCerca(pos);
	if (pos.dist( h_pos ) < 250)
			Fire(h_pos);		

	Punto2D pant_pos = current->ScreenPos(pos);
	if (pant_pos.Y() > Gra_ScreenHeight()/2.00)
		vel = Punto2D(-3*dir, -2);
	else
		vel = Punto2D(3*dir, -2);
	vel /= vel.Norma();
	vel *= 1.5;
	pos += vel*Gra_SpeedFactor();
}
